/*
 * playlistaspect.h
 *
 *  Created on: 15.04.2010
 *      Author: darkstar
 */

#ifndef PLAYLISTASPECT_H_
#define PLAYLISTASPECT_H_

#ifdef QT4
#include <QObject>
#include <QHash>
#include <QList>
#else
#include <qobject.h>
#include <qdict.h>
#include <qlist.h>
#endif

#include "playlist.h"

class PlayListAspectExtension;

#ifdef QT4
typedef QList<Media *> MediaList;
typedef QList<PlayListAspectExtension *> PlayListAspectExtensionList;
#else
typedef QList<Media> MediaList;
typedef QList<PlayListAspectExtension> PlayListAspectExtensionList;
#endif

class PlayListAspect : public QObject
{
    Q_OBJECT
public:
    PlayListAspect(PlayList *playList, const QString aspectName);
    virtual ~PlayListAspect();

    PlayList *playList() { return playList_; }
    const QString &name() { return aspectName_; }
    const QString &prefix() { return prefix_; }

    PlayListAspectExtension *appendExtension(const QString &className);
    void appendExtension(PlayListAspectExtension *extension);
    void removeExtension(PlayListAspectExtension *extension);

    void loadFromDatabase();
    void saveToDatabase();

    enum ExecutionReason
    {
        SelfChanged = 0,
        UpstreamChanged = 1
    };

public slots:
    void update();

protected:
    friend class PlayListAspectExtension;

    void beginUpdate();
    void endUpdate();

    void requestingUpdate(PlayListAspectExtension *starter);
    void runPreCondition(bool &stopExecution);
    void runPostCondition();

private:
    PlayList *playList_;
    QString aspectName_;
    QString prefix_;

    int updateCount_;

    PlayListAspectExtensionList extensions_;
    PlayListAspectExtension *filterExtensionStart_;
};

class PlayListAspectExtension
{
public:
    PlayListAspectExtension() : playListAspect_(NULL) {}
    virtual ~PlayListAspectExtension() { }

    PlayListAspect *playListAspect() { return playListAspect_; }

    void update();

    virtual void reset();
    virtual void resetFilter();
    virtual const QString outputViewSource() = 0;

    virtual bool isActive();
    virtual bool isFiltering();

    bool isGraphFiltered();

    virtual const QString identifier() { return outputViewSource(); }
    virtual void saveExtensionState(QStringList &dst);
    virtual void loadExtensionState(QStringList &src);

protected:
    friend class PlayListAspect;

    void setPlayListAspect(PlayListAspect *playListAspect);

    virtual void attachedToAspect() {}
    virtual void removedFromAspect() {}

    void setExtensionList(PlayListAspectExtensionList *extensionList) { extensionList_ = extensionList; }
    PlayListAspectExtensionList *extenstionList() { return extensionList_; }

    QString inputViewSource();

    virtual void execute(PlayListAspect::ExecutionReason reason, PlayListAspectExtension *starter) = 0;

    void continueWithNextAspectExtension();

    PlayListAspectExtension *previousAspectExtension();
    PlayListAspectExtension *nextAspectExtension();

    virtual void extensionGraphExecutionStarted(bool &stopIt);
    virtual void extensionGraphExecutionFinished();

private:
    PlayListAspect *playListAspect_;
    PlayListAspectExtensionList *extensionList_;
};

class AbstractPlayListAspectExtensionFactory
{
public:
    virtual ~AbstractPlayListAspectExtensionFactory() {}
    virtual QString internalClassName() const = 0;
    virtual PlayListAspectExtension *createInstance() = 0;
};

#ifdef QT4
typedef QList<AbstractPlayListAspectExtensionFactory *> PlayListAspectExtensionFactoryList;
#else
typedef QList<AbstractPlayListAspectExtensionFactory> PlayListAspectExtensionFactoryList;
#endif

class PlayListAspectExtensionRegistry
{
public:
    virtual ~PlayListAspectExtensionRegistry() {}

    static PlayListAspectExtensionRegistry& singleton();
#	define globalPlayListAspectExtensionRegistry PlayListAspectExtensionRegistry::singleton()

    PlayListAspectExtension *createNewInstance(const QString &className);

    void registerFactory(AbstractPlayListAspectExtensionFactory *factory);
    AbstractPlayListAspectExtensionFactory *findFactory(const QString &className);

    PlayListAspectExtensionFactoryList &factories() { return factories_; }

private:
    PlayListAspectExtensionFactoryList factories_;
#ifdef QT4
    QHash<QString, AbstractPlayListAspectExtensionFactory *> factoriesMap_;
#else
    QDict<AbstractPlayListAspectExtensionFactory> factoriesMap_;
#endif
};

template <class AspectClass>
class PlayListAspectExtensionFactory : public AbstractPlayListAspectExtensionFactory
{
public:
    PlayListAspectExtensionFactory(QString className) : className_(className) { globalPlayListAspectExtensionRegistry.registerFactory(this); }
    virtual QString internalClassName() const { return className_; }
    virtual PlayListAspectExtension *createInstance() { return new AspectClass(); }
protected:
    QString className_;
};

#endif /* PLAYLISTASPECT_H_ */
